@@ -12,36 +12,24 @@ import numpy as np  # type: ignore
 import pandas as pd  # type: ignore
 
 from worklog.breaks import AutoBreak
-from worklog.constants import (
-    COL_CATEGORY,
-    COL_COMMIT_DATETIME,
-    COL_LOG_DATETIME,
-    COL_TASK_IDENTIFIER,
-    COL_TYPE,
-    DEFAULT_LOGGER_NAME,
-    LOCAL_TIMEZONE,
-    TOKEN_SESSION,
-    TOKEN_START,
-    TOKEN_STOP,
-    TOKEN_TASK,
-)
-from worklog.utils import (
-    calc_log_time,
-    check_order_session,
-    empty_df_from_schema,
-    extract_intervals,
-    format_timedelta,
-    get_active_task_ids,
-    get_all_task_ids,
-    get_all_task_ids_with_duration,
-    get_datetime_cols_from_schema,
-    get_pager,
-    sentinel_datetime,
-    format_numpy_timedelta,
+import worklog.constants as wc
+from worklog.utils.pager import get_pager
+from worklog.utils.time import calc_log_time, extract_date_and_time
+from worklog.utils.schema import empty_df_from_schema, get_datetime_cols_from_schema
+from worklog.utils.formatting import format_timedelta
+from worklog.utils.tasks import (
     calc_task_durations,
+    extract_intervals,
+    get_active_task_ids,
+    get_all_task_ids_with_duration,
+)
+from worklog.utils.session import (
+    check_order_session,
+    sentinel_datetime,
+    is_active_session,
 )
 
-logger = logging.getLogger(DEFAULT_LOGGER_NAME)
+logger = logging.getLogger(wc.DEFAULT_LOGGER_NAME)
 
 
 class Log(object):
@@ -52,11 +40,11 @@ class Log(object):
     _log_fp: Optional[str] = None
     _separator: Optional[str] = None
     _schema: List[Tuple[str, str]] = [
-        (COL_COMMIT_DATETIME, "datetime64[ns]",),
-        (COL_LOG_DATETIME, "datetime64[ns]",),
-        (COL_CATEGORY, "object",),
-        (COL_TYPE, "object",),
-        (COL_TASK_IDENTIFIER, "object",),
+        (wc.COL_COMMIT_DATETIME, "datetime64[ns]",),
+        (wc.COL_LOG_DATETIME, "datetime64[ns]",),
+        (wc.COL_CATEGORY, "object",),
+        (wc.COL_TYPE, "object",),
+        (wc.COL_TASK_IDENTIFIER, "object",),
     ]
 
     # Error messages
@@ -102,22 +90,24 @@ class Log(object):
     def list_tasks(self):
         """List all known tasks, i.e. tasks that have been used previously
         and are stored in the logfile."""
-        task_df = self._log_df[self._log_df[COL_CATEGORY] == TOKEN_TASK]
+        task_df = self._log_df[self._log_df[wc.COL_CATEGORY] == wc.TOKEN_TASK]
         sys.stdout.write("These tasks are listed in the log:\n")
-        for task_id in sorted(task_df[COL_TASK_IDENTIFIER].unique()):
+        for task_id in sorted(task_df[wc.COL_TASK_IDENTIFIER].unique()):
             sys.stdout.write(f"{task_id}\n")
 
-    def log(self, n: int, use_pager: bool, filter_category: List[str]) -> None:
+    def log(
+        self, n: int, use_pager: bool, filter_category: Optional[List[str]]
+    ) -> None:
         """Display the content of the logfile."""
         if self._log_df.shape[0] == 0:
             sys.stdout.write("No data available\n")
             return
 
-        fields = ["date", "time", COL_CATEGORY, COL_TYPE, COL_TASK_IDENTIFIER]
+        fields = ["date", "time", wc.COL_CATEGORY, wc.COL_TYPE, wc.COL_TASK_IDENTIFIER]
         df = self._log_df[fields].iloc[::-1]  # sort in reverse (latest first)
-        df[COL_TASK_IDENTIFIER] = df[COL_TASK_IDENTIFIER].fillna("-")
+        df[wc.COL_TASK_IDENTIFIER] = df[wc.COL_TASK_IDENTIFIER].fillna("-")
         if filter_category:
-            df = df[df[COL_CATEGORY] == filter_category]
+            df = df[df[wc.COL_CATEGORY] == filter_category]
         if n > 0:
             df = df.head(n=n)
         if not use_pager:
@@ -138,10 +128,10 @@ class Log(object):
     def report(self, date_from: datetime, date_to: datetime):
         """Generate a daily, weekly, monthly and task based report based on
         the content in the logfile."""
-        session_mask = self._log_df[COL_CATEGORY] == TOKEN_SESSION
-        task_mask = self._log_df[COL_CATEGORY] == TOKEN_TASK
-        time_mask = (self._log_df[COL_LOG_DATETIME] >= date_from) & (
-            self._log_df[COL_LOG_DATETIME] < date_to
+        session_mask = self._log_df[wc.COL_CATEGORY] == wc.TOKEN_SESSION
+        task_mask = self._log_df[wc.COL_CATEGORY] == wc.TOKEN_TASK
+        time_mask = (self._log_df[wc.COL_LOG_DATETIME] >= date_from) & (
+            self._log_df[wc.COL_LOG_DATETIME] < date_to
         )
 
         # Day aggregation
@@ -149,10 +139,14 @@ class Log(object):
         df_day["break"] = df_day["agg_time"].map(self.auto_break.get_duration)
 
         # Week aggregation
-        df_week = df_day.set_index(COL_LOG_DATETIME).resample("W").sum().reset_index()
+        df_week = (
+            df_day.set_index(wc.COL_LOG_DATETIME).resample("W").sum().reset_index()
+        )
 
         # Month aggregration
-        df_month = df_day.set_index(COL_LOG_DATETIME).resample("M").sum().reset_index()
+        df_month = (
+            df_day.set_index(wc.COL_LOG_DATETIME).resample("M").sum().reset_index()
+        )
 
         for df in (df_day, df_week, df_month):
             df["agg_time_bookable"] = df["agg_time"] - df["break"]
@@ -160,7 +154,7 @@ class Log(object):
         # Task aggregation
         df_tasks = self._aggregate_tasks(time_mask & task_mask)
 
-        print_cols = [COL_LOG_DATETIME, "agg_time"]
+        print_cols = [wc.COL_LOG_DATETIME, "agg_time"]
         print_cols_labels = ["Date", "Total time"]
         if self.auto_break.active:
             print_cols += ["break", "agg_time_bookable"]
@@ -169,9 +163,9 @@ class Log(object):
         def _formatters(date_type: str = "M"):
             date_max_len = len("2000-01") if date_type == "M" else len("2000-01-01")
             return {
-                COL_LOG_DATETIME: lambda v: str(v.date())[:date_max_len],
-                "agg_time": format_numpy_timedelta,
-                "agg_time_bookable": format_numpy_timedelta,
+                wc.COL_LOG_DATETIME: lambda v: str(v.date())[:date_max_len],
+                "agg_time": format_timedelta,
+                "agg_time_bookable": format_timedelta,
             }
 
         self._print_aggregation(
@@ -188,7 +182,7 @@ class Log(object):
             "day", df_day, print_cols, print_cols_labels, formatters=_formatters("D")
         )
 
-        print_cols = [COL_TASK_IDENTIFIER, "agg_time"]
+        print_cols = [wc.COL_TASK_IDENTIFIER, "agg_time"]
         print_cols_labels = ["Task name", "Total time"]
         self._print_aggregation(
             "tasks",
@@ -217,14 +211,17 @@ class Log(object):
                 sys.stdout.write(self._err_msg_log_data_missing_for_date_short)
             return
 
-        is_active = self._is_active(df_day)
+        is_active = is_active_session(df_day)
         logger.debug(f"Is active: {is_active}")
 
         df_day = self._add_sentinel(query_date, df_day)
         facts = self._calc_facts(df_day, hours_target, hours_max)
 
-        all_touched_tasks = get_all_task_ids_with_duration(self._log_df, query_date)
-        active_tasks = get_active_task_ids(self._log_df, query_date)
+        date_mask = self._log_df["date"] == query_date
+        task_mask = self._log_df[wc.COL_CATEGORY] == wc.TOKEN_TASK
+        sel_task_mask = date_mask & task_mask
+        all_touched_tasks = get_all_task_ids_with_duration(self._log_df[sel_task_mask])
+        active_tasks = get_active_task_ids(self._log_df[sel_task_mask])
 
         lines = [
             ("Status", "Tracking {status}"),
@@ -263,14 +260,15 @@ class Log(object):
     def stop_active_tasks(self, log_dt: datetime):
         """Stop all active tasks by commiting changes to the logfile."""
         query_date = log_dt.date()
-        active_task_ids = get_active_task_ids(self._log_df, query_date)
+        date_mask = self._log_df["date"] == query_date
+        active_task_ids = get_active_task_ids(self._log_df[date_mask])
         for task_id in active_task_ids:
-            self._commit(TOKEN_TASK, TOKEN_STOP, log_dt, identifier=task_id)
+            self._commit(wc.TOKEN_TASK, wc.TOKEN_STOP, log_dt, identifier=task_id)
 
     def task_report(self, task_id):
         """Generate a report of a given task."""
-        task_mask = self._log_df[COL_CATEGORY] == TOKEN_TASK
-        task_id_mask = self._log_df[COL_TASK_IDENTIFIER] == task_id
+        task_mask = self._log_df[wc.COL_CATEGORY] == wc.TOKEN_TASK
+        task_id_mask = self._log_df[wc.COL_TASK_IDENTIFIER] == task_id
         mask = task_mask & task_id_mask
         task_df = self._log_df[mask]
 
@@ -330,25 +328,14 @@ class Log(object):
                 parse_dates=date_cols,
                 header=None,
                 names=header,
-            ).sort_values(by=[COL_LOG_DATETIME])
+            ).sort_values(by=[wc.COL_LOG_DATETIME])
         except pd.errors.EmptyDataError:
             self._log_df = empty_df_from_schema(self._schema)
 
         self._log_df = pd.concat(
-            [self._log_df, self._extract_date_and_time(self._log_df)], axis=1
+            [self._log_df, extract_date_and_time(self._log_df)], axis=1
         )
 
-    def _extract_date_and_time(
-        self, df: pd.DataFrame, source_col: str = COL_LOG_DATETIME
-    ) -> pd.DataFrame:
-        """
-        Extracts date and time information from a given pandas DataFrame.
-        By default the source column is `log_dt`.
-        """
-        date: pd.Series = df[source_col].apply(lambda x: x.date)
-        time: pd.Series = df[source_col].apply(lambda x: x.time)
-        return pd.DataFrame(dict(date=date, time=time),)
-
     def _persist(self, df: pd.DataFrame, mode="a") -> None:
         cols = [col for col, _ in self._schema]
         df[cols].to_csv(
@@ -363,16 +350,17 @@ class Log(object):
         identifier: str = None,
         force: bool = False,
     ) -> None:
-        if type_ not in [TOKEN_START, TOKEN_STOP]:
+        if type_ not in [wc.TOKEN_START, wc.TOKEN_STOP]:
             raise ValueError(
-                f'Type must be one of {", ".join([TOKEN_START, TOKEN_STOP])}'
+                f'Type must be one of {", ".join([wc.TOKEN_START, wc.TOKEN_STOP])}'
             )
 
         commit_dt = datetime.now(timezone.utc).astimezone().replace(microsecond=0)
 
         # Test if there are running tasks
-        if category == TOKEN_SESSION:
-            active_tasks = get_active_task_ids(self._log_df, log_dt.date())
+        if category == wc.TOKEN_SESSION:
+            date_mask = self._log_df["date"] == log_dt.date()
+            active_tasks = get_active_task_ids(self._log_df[date_mask])
             if len(active_tasks) > 0:
                 if not force:
                     msg = self._err_msg_commit_active_tasks.format(
@@ -382,7 +370,7 @@ class Log(object):
                     sys.exit(1)
                 else:
                     for task_id in active_tasks:
-                        self._commit(TOKEN_TASK, TOKEN_STOP, log_dt, task_id)
+                        self._commit(wc.TOKEN_TASK, wc.TOKEN_STOP, log_dt, task_id)
 
         cols = [col for col, _ in self._schema]
         values = [
@@ -394,7 +382,7 @@ class Log(object):
         ]
 
         record = pd.DataFrame(dict(zip(cols, values)), index=[0],)
-        record_t = pd.concat([record, self._extract_date_and_time(record)], axis=1)
+        record_t = pd.concat([record, extract_date_and_time(record)], axis=1)
 
         # append record to in-memory log
         self._log_df = pd.concat((self._log_df, record_t))
@@ -403,18 +391,7 @@ class Log(object):
 
         # Because we allow for time offsets sorting is not guaranteed at this point.
         # Update sorting of values in-memory.
-        self._log_df = self._log_df.sort_values(by=[COL_LOG_DATETIME])
-
-    def _is_active(self, df: pd.DataFrame):
-        """
-        Returns True if the last entry in a given pandas DataFrame has 
-        `type == 'start'`.
-
-        Note: This method does not check for anything else. If this should
-        just be applied to a single category, make sure to filter the pandas
-        DataFrame first.
-        """
-        return df.iloc[-1][COL_TYPE] == TOKEN_START if df.shape[0] > 0 else False
+        self._log_df = self._log_df.sort_values(by=[wc.COL_LOG_DATETIME])
 
     def _check_nonempty_or_exit(self, fmt: Optional[str]):
         """
@@ -431,8 +408,8 @@ class Log(object):
     def _filter_date_category_limit_cols(
         self,
         query_date: date,
-        filter_category: str = TOKEN_SESSION,
-        columns: List[str] = [COL_LOG_DATETIME, COL_TYPE],
+        filter_category: str = wc.TOKEN_SESSION,
+        columns: List[str] = [wc.COL_LOG_DATETIME, wc.COL_TYPE],
     ):
         """
         Filters the worklog DataFrame by query date and category.
@@ -449,15 +426,15 @@ class Log(object):
         return df
 
     def _add_sentinel(self, query_date: date, df: pd.DataFrame):
-        is_active = self._is_active(df)
+        is_active = is_active_session(df)
         ret = df
         if is_active:
             sdt = sentinel_datetime(query_date)
             # attach another row with the current time
             sentinel_df = pd.DataFrame(
                 {
-                    COL_LOG_DATETIME: pd.to_datetime(sdt.isoformat()),
-                    COL_TYPE: TOKEN_STOP,
+                    wc.COL_LOG_DATETIME: pd.to_datetime(sdt.isoformat()),
+                    wc.COL_TYPE: wc.TOKEN_STOP,
                 },
                 index=[0],
             )
@@ -466,9 +443,9 @@ class Log(object):
         return ret
 
     def _calc_facts(self, df: pd.DataFrame, hours_target: float, hours_max: float):
-        shifted_dt = df[COL_LOG_DATETIME].shift(1)
-        stop_mask = df[COL_TYPE] == TOKEN_STOP
-        total_time = (df[stop_mask][COL_LOG_DATETIME] - shifted_dt[stop_mask]).sum()
+        shifted_dt = df[wc.COL_LOG_DATETIME].shift(1)
+        stop_mask = df[wc.COL_TYPE] == wc.TOKEN_STOP
+        total_time = (df[stop_mask][wc.COL_LOG_DATETIME] - shifted_dt[stop_mask]).sum()
         total_time_str = format_timedelta(total_time)
 
         break_duration = self.auto_break.get_duration(total_time)
@@ -511,18 +488,18 @@ class Log(object):
 
     def _aggregate_base(self, mask, keep_cols: List[str] = []):
         df = self._log_df[mask]
-        df = df.sort_values([COL_LOG_DATETIME, COL_TYPE])
-        shifted_dt = df[COL_LOG_DATETIME].shift(1)
-        stop_mask = df[COL_TYPE] == TOKEN_STOP
-        agg_time = df[stop_mask][COL_LOG_DATETIME] - shifted_dt[stop_mask]
-        ret = df[stop_mask][[COL_LOG_DATETIME] + keep_cols]
+        df = df.sort_values([wc.COL_LOG_DATETIME, wc.COL_TYPE])
+        shifted_dt = df[wc.COL_LOG_DATETIME].shift(1)
+        stop_mask = df[wc.COL_TYPE] == wc.TOKEN_STOP
+        agg_time = df[stop_mask][wc.COL_LOG_DATETIME] - shifted_dt[stop_mask]
+        ret = df[stop_mask][[wc.COL_LOG_DATETIME] + keep_cols]
         ret["agg_time"] = agg_time
         return ret
 
     def _aggregate_time(self, mask, resample="D"):
         df = self._aggregate_base(mask, keep_cols=["date"])
         df_day = (
-            df.set_index(COL_LOG_DATETIME)
+            df.set_index(wc.COL_LOG_DATETIME)
             .resample(resample)
             .sum()
             .reset_index()
@@ -533,12 +510,12 @@ class Log(object):
     def _aggregate_tasks(self, mask):
         df = calc_task_durations(
             self._log_df[mask],
-            keep_cols=[COL_LOG_DATETIME, COL_TASK_IDENTIFIER, "time"],
+            keep_cols=[wc.COL_LOG_DATETIME, wc.COL_TASK_IDENTIFIER, "time"],
         )
         df.rename(columns={"time": "agg_time"}, inplace=True)
         return (
-            df.set_index(COL_LOG_DATETIME)
-            .groupby(COL_TASK_IDENTIFIER)
+            df.set_index(wc.COL_LOG_DATETIME)
+            .groupby(wc.COL_TASK_IDENTIFIER)
             .sum()
             .reset_index()
         )
